@@ -12,7 +12,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import hashlib
 import io
 import os
 import re
@@ -24,11 +23,11 @@ import flask
 import jinja2
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils.secretutils import md5
 import webob
 from werkzeug import exceptions
 
 from octavia.amphorae.backends.agent.api_server import haproxy_compatibility
-from octavia.amphorae.backends.agent.api_server import osutils
 from octavia.amphorae.backends.agent.api_server import util
 from octavia.common import constants as consts
 from octavia.common import utils as octavia_utils
@@ -56,7 +55,7 @@ SYSTEMD_TEMPLATE = JINJA_ENV.get_template(SYSTEMD_CONF)
 class Wrapped(object):
     def __init__(self, stream_):
         self.stream = stream_
-        self.hash = hashlib.md5()  # nosec
+        self.hash = md5(usedforsecurity=False)  # nosec
 
     def read(self, line):
         block = self.stream.read(line)
@@ -73,20 +72,18 @@ class Wrapped(object):
 
 class Loadbalancer(object):
 
-    def __init__(self):
-        self._osutils = osutils.BaseOS.get_os_util()
-
     def get_haproxy_config(self, lb_id):
         """Gets the haproxy config
 
         :param listener_id: the id of the listener
         """
         self._check_lb_exists(lb_id)
-        with open(util.config_path(lb_id), 'r') as file:
+        with open(util.config_path(lb_id), 'r', encoding='utf-8') as file:
             cfg = file.read()
             resp = webob.Response(cfg, content_type='text/plain')
             resp.headers['ETag'] = (
-                hashlib.md5(octavia_utils.b(cfg)).hexdigest())  # nosec
+                md5(octavia_utils.b(cfg),
+                    usedforsecurity=False).hexdigest())  # nosec
             return resp
 
     def upload_haproxy_config(self, amphora_id, lb_id):
@@ -198,7 +195,6 @@ class Loadbalancer(object):
                                       respawn_interval),
                     amphora_netns=consts.AMP_NETNS_SVC_PREFIX,
                     amphora_nsname=consts.AMPHORA_NAMESPACE,
-                    HasIFUPAll=self._osutils.has_ifup_all(),
                     haproxy_major_version=hap_major,
                     haproxy_minor_version=hap_minor
                 )
@@ -235,12 +231,11 @@ class Loadbalancer(object):
                 details="Unknown action: {0}".format(action)), status=400)
 
         self._check_lb_exists(lb_id)
+        is_vrrp = (CONF.controller_worker.loadbalancer_topology ==
+                   consts.TOPOLOGY_ACTIVE_STANDBY)
 
-        # Since this script should be created at LB create time
-        # we can check for this path to see if VRRP is enabled
-        # on this amphora and not write the file if VRRP is not in use
-        if os.path.exists(util.keepalived_check_script_path()):
-            self.vrrp_check_script_update(lb_id, action)
+        if is_vrrp:
+            util.vrrp_check_script_update(lb_id, action)
 
         # HAProxy does not start the process when given a reload
         # so start it if haproxy is not already running
@@ -262,6 +257,14 @@ class Loadbalancer(object):
                 return webob.Response(json=dict(
                     message="Error {0}ing haproxy".format(action),
                     details=e.output), status=500)
+
+        # If we are not in active/standby we need to send an IP
+        # advertisement (GARP or NA). Keepalived handles this for
+        # active/standby load balancers.
+        if not is_vrrp and action in [consts.AMP_ACTION_START,
+                                      consts.AMP_ACTION_RELOAD]:
+            util.send_vip_advertisements(lb_id)
+
         if action in [consts.AMP_ACTION_STOP,
                       consts.AMP_ACTION_RELOAD]:
             return webob.Response(json=dict(
@@ -307,7 +310,7 @@ class Loadbalancer(object):
         # we can check for this path to see if VRRP is enabled
         # on this amphora and not write the file if VRRP is not in use
         if os.path.exists(util.keepalived_check_script_path()):
-            self.vrrp_check_script_update(
+            util.vrrp_check_script_update(
                 lb_id, action=consts.AMP_ACTION_STOP)
 
         # delete the ssl files
@@ -357,7 +360,7 @@ class Loadbalancer(object):
 
         Currently type==SSL is also not detected
         """
-        listeners = list()
+        listeners = []
 
         for lb in util.get_loadbalancers():
             stats_socket, listeners_on_lb = util.parse_haproxy_file(lb)
@@ -406,11 +409,12 @@ class Loadbalancer(object):
                 details="No certificate with filename: {f}".format(
                     f=filename)), status=404)
 
-        with open(cert_path, 'r') as crt_file:
+        with open(cert_path, 'r', encoding='utf-8') as crt_file:
             cert = crt_file.read()
-            md5 = hashlib.md5(octavia_utils.b(cert)).hexdigest()  # nosec
-            resp = webob.Response(json=dict(md5sum=md5))
-            resp.headers['ETag'] = md5
+            md5sum = md5(octavia_utils.b(cert),
+                         usedforsecurity=False).hexdigest()  # nosec
+            resp = webob.Response(json=dict(md5sum=md5sum))
+            resp.headers['ETag'] = md5sum
             return resp
 
     def delete_certificate(self, lb_id, filename):
@@ -424,7 +428,8 @@ class Loadbalancer(object):
             if os.path.exists(
                     os.path.join('/proc', util.get_haproxy_pid(lb_id))):
                 # Check if the listener is disabled
-                with open(util.config_path(lb_id), 'r') as file:
+                with open(util.config_path(lb_id),
+                          'r', encoding='utf-8') as file:
                     cfg = file.read()
                     m = re.findall('^frontend (.*)$', cfg, re.MULTILINE)
                     return m or []
@@ -455,22 +460,6 @@ class Loadbalancer(object):
     def _cert_file_path(self, lb_id, filename):
         return os.path.join(self._cert_dir(lb_id), filename)
 
-    def vrrp_check_script_update(self, lb_id, action):
-        lb_ids = util.get_loadbalancers()
-        if action == consts.AMP_ACTION_STOP:
-            lb_ids.remove(lb_id)
-        args = []
-        for lbid in lb_ids:
-            args.append(util.haproxy_sock_path(lbid))
-
-        if not os.path.exists(util.keepalived_dir()):
-            os.makedirs(util.keepalived_dir())
-            os.makedirs(util.keepalived_check_scripts_dir())
-
-        cmd = 'haproxy-vrrp-check {args}; exit $?'.format(args=' '.join(args))
-        with open(util.haproxy_check_script_path(), 'w') as text_file:
-            text_file.write(cmd)
-
     def _check_haproxy_status(self, lb_id):
         if os.path.exists(util.pid_path(lb_id)):
             if os.path.exists(
