@@ -16,6 +16,7 @@ import os
 import re
 
 import jinja2
+from octavia_lib.common import constants as lib_consts
 
 from octavia.common.config import cfg
 from octavia.common import constants
@@ -26,6 +27,7 @@ PROTOCOL_MAP = {
     constants.PROTOCOL_HTTP: 'http',
     constants.PROTOCOL_HTTPS: 'tcp',
     constants.PROTOCOL_PROXY: 'proxy',
+    lib_consts.PROTOCOL_PROXYV2: 'proxy',
     constants.PROTOCOL_TERMINATED_HTTPS: 'http'
 }
 
@@ -98,6 +100,8 @@ class JinjaTemplater(object):
         # Is it newer than haproxy 1.5?
         if not (int(haproxy_versions[0]) < 2 and int(haproxy_versions[1]) < 6):
             feature_compatibility[constants.HTTP_REUSE] = True
+        if not (int(haproxy_versions[0]) < 2 and int(haproxy_versions[1]) < 9):
+            feature_compatibility[constants.POOL_ALPN] = True
 
         return self.render_loadbalancer_obj(
             host_amphora, listeners, tls_certs=tls_certs,
@@ -167,7 +171,7 @@ class JinjaTemplater(object):
                  CONF.amphora_agent.administrative_log_facility,
              'user_log_facility': CONF.amphora_agent.user_log_facility,
              'connection_logging': self.connection_logging},
-            constants=constants)
+            constants=constants, lib_consts=lib_consts)
 
     def _transform_loadbalancer(self, host_amphora, loadbalancer, listeners,
                                 tls_certs, feature_compatibility):
@@ -177,7 +181,7 @@ class JinjaTemplater(object):
         """
         listener_transforms = []
         for listener in listeners:
-            if listener.protocol == constants.PROTOCOL_UDP:
+            if listener.protocol in constants.LVS_PROTOCOLS:
                 continue
             listener_transforms.append(self._transform_listener(
                 listener, tls_certs, feature_compatibility, loadbalancer))
@@ -197,7 +201,7 @@ class JinjaTemplater(object):
         # listeners' connection limits.
         connection_limit_sum = 0
         for listener in listeners:
-            if listener.protocol == constants.PROTOCOL_UDP:
+            if listener.protocol in constants.LVS_PROTOCOLS:
                 continue
             if listener.connection_limit and listener.connection_limit > -1:
                 connection_limit_sum += listener.connection_limit
@@ -283,9 +287,15 @@ class JinjaTemplater(object):
                     os.path.join(self.base_crt_dir, loadbalancer.id,
                                  tls_certs[listener.client_crl_container_id]))
 
-        if (listener.protocol == constants.PROTOCOL_TERMINATED_HTTPS and
-                listener.tls_ciphers is not None):
-            ret_value['tls_ciphers'] = listener.tls_ciphers
+        tls_enabled = False
+        if listener.protocol == constants.PROTOCOL_TERMINATED_HTTPS:
+            tls_enabled = True
+            if listener.tls_ciphers is not None:
+                ret_value['tls_ciphers'] = listener.tls_ciphers
+            if listener.tls_versions is not None:
+                ret_value['tls_versions'] = listener.tls_versions
+            if listener.alpn_protocols is not None:
+                ret_value['alpn_protocols'] = ",".join(listener.alpn_protocols)
 
         pools = []
         pool_gen = (pool for pool in listener.pools if
@@ -295,7 +305,7 @@ class JinjaTemplater(object):
             if tls_certs is not None and tls_certs.get(pool.id):
                 kwargs = {'pool_tls_certs': tls_certs.get(pool.id)}
             pools.append(self._transform_pool(
-                pool, feature_compatibility, **kwargs))
+                pool, feature_compatibility, tls_enabled, **kwargs))
         ret_value['pools'] = pools
         policy_gen = (policy for policy in listener.l7policies if
                       policy.provisioning_status != constants.PENDING_DELETE)
@@ -306,21 +316,27 @@ class JinjaTemplater(object):
                     break
 
         l7policies = [self._transform_l7policy(
-                      x, feature_compatibility, tls_certs)
+                      x, feature_compatibility, tls_enabled, tls_certs)
                       for x in policy_gen]
         ret_value['l7policies'] = l7policies
         return ret_value
 
     def _transform_pool(self, pool, feature_compatibility,
-                        pool_tls_certs=None):
+                        listener_tls_enabled, pool_tls_certs=None):
         """Transforms a pool into an object that will
 
             be processed by the templating system
         """
+        proxy_protocol_version = None
+        if pool.protocol == constants.PROTOCOL_PROXY:
+            proxy_protocol_version = 1
+        if pool.protocol == lib_consts.PROTOCOL_PROXYV2:
+            proxy_protocol_version = 2
         ret_value = {
             'id': pool.id,
             'protocol': PROTOCOL_MAP[pool.protocol],
-            'proxy_protocol': pool.protocol == constants.PROTOCOL_PROXY,
+            'proxy_protocol': proxy_protocol_version,
+            'listener_tls_enabled': listener_tls_enabled,
             'lb_algorithm': BALANCE_MAP.get(pool.lb_algorithm, 'roundrobin'),
             'members': [],
             'health_monitor': '',
@@ -351,8 +367,14 @@ class JinjaTemplater(object):
         if (pool.tls_certificate_id and pool_tls_certs and
                 pool_tls_certs.get('client_cert')):
             ret_value['client_cert'] = pool_tls_certs.get('client_cert')
-        if pool.tls_enabled is True and pool.tls_ciphers is not None:
-            ret_value['tls_ciphers'] = pool.tls_ciphers
+        if pool.tls_enabled is True:
+            if pool.tls_ciphers is not None:
+                ret_value['tls_ciphers'] = pool.tls_ciphers
+            if pool.tls_versions is not None:
+                ret_value['tls_versions'] = pool.tls_versions
+            if (pool.alpn_protocols is not None and
+                    feature_compatibility.get(constants.POOL_ALPN, False)):
+                ret_value['alpn_protocols'] = ",".join(pool.alpn_protocols)
         if (pool.ca_tls_certificate_id and pool_tls_certs and
                 pool_tls_certs.get('ca_cert')):
             ret_value['ca_cert'] = pool_tls_certs.get('ca_cert')
@@ -417,7 +439,7 @@ class JinjaTemplater(object):
         }
 
     def _transform_l7policy(self, l7policy, feature_compatibility,
-                            tls_certs=None):
+                            listener_tls_enabled, tls_certs=None):
         """Transforms an L7 policy into an object that will
 
             be processed by the templating system
@@ -438,7 +460,8 @@ class JinjaTemplater(object):
                 kwargs = {'pool_tls_certs':
                           tls_certs.get(l7policy.redirect_pool.id)}
             ret_value['redirect_pool'] = self._transform_pool(
-                l7policy.redirect_pool, feature_compatibility, **kwargs)
+                l7policy.redirect_pool, feature_compatibility,
+                listener_tls_enabled, **kwargs)
         else:
             ret_value['redirect_pool'] = None
         if (l7policy.action in [constants.L7POLICY_ACTION_REDIRECT_TO_URL,
