@@ -58,11 +58,12 @@ def update_stats_on_done(stats, fut):
 
 class HealthManager(object):
     def __init__(self, exit_event):
-        if CONF.api_settings.default_provider_driver == constants.AMPHORAV2:
-            self.cw = cw2.ControllerWorker()
-        else:
+        if CONF.api_settings.default_provider_driver == constants.AMPHORAV1:
             self.cw = cw1.ControllerWorker()
+        else:
+            self.cw = cw2.ControllerWorker()
         self.threads = CONF.health_manager.failover_threads
+        # pylint: disable=consider-using-with
         self.executor = futures.ThreadPoolExecutor(max_workers=self.threads)
         self.amp_repo = repo.AmphoraRepository()
         self.amp_health_repo = repo.AmphoraHealthRepository()
@@ -136,7 +137,7 @@ class HealthManager(object):
 
             LOG.info("Stale amphora's id is: %s", amp_health.amphora_id)
             fut = self.executor.submit(
-                self.cw.failover_amphora, amp_health.amphora_id)
+                self.cw.failover_amphora, amp_health.amphora_id, reraise=True)
             fut.add_done_callback(
                 functools.partial(update_stats_on_done, stats)
             )
